{ VESA16 V2.1 Slovak                                                         }
{ For english comments see SVGA256.PAS                                       }
{ Unit pre pracu s 16-farebnou grafikou                                      }
{ Potrebuje nasledovne subory:                                               }
{  VESA16.BGI   -  Borland Graphics Interface pre VESA karty                 }
{ (c) Matus Horvath, 1997                                                    }
unit VESA16;
{-[ INTERFACE ]----------------------------------------------------/////-----}
 interface
{-----[ Typ DacPalette256 pre pracu s paletou ]--------------------/////-----}
  type DacPalette256 = array[0..255] of array[0..2] of Byte;
{-----[ Typ VesaInfo - informacie o grafickej karte ]--------------/////-----}
       VESAInfo=record
                 VESAVer_Hi:byte;
                 VESAVer_Lo:byte;
                 Manufacturer:string;
                 Modes:byte;
                end;

{-----[ Konstanty pre inicializaciu grafiky ]----------------------/////-----}
  const SVGA800x600x16    = 0;  {   800x 600x16 }
        S8615             = 0;
        SVGA1024x768x16   = 1;  {  1024x 768x16 }
        S1716             = 1;
        SVGA1280x1024x16  = 2;  {  1280x1024x16 }
        S1116             = 2;
{-----[ Konstanty pre cesty k suborom BGI a CHR ]------------------/////-----}
        Bgi    = 'C:\TP\BGI\';  { Adresar BGI  }
        TP     = 'C:\TP\';      { Adresar TP   }
{-----[ Premenne ]-------------------------------------------------/////-----}
  var VESA:integer;             { Cislo drivera pre InitGraph           }
      MaxVESAMode:integer;      { Maximalny mod detekovany InitGraph-om }
      SVGAInfo:VESAInfo;

{-----[ Inicializacia ]--------------------------------------------/////-----}
{-------[ Procedura INITIALIZE kompatibilna s VESA16 1.x ]---------/////-----}
  procedure Initialize(gm:integer;pd:string);
{-------[ Procedury pre rychlu inicializaciu ]---------------------/////-----}
  procedure I8616;    {  800x 600x16 }
  procedure I1716;    { 1024x 768x16 }
  procedure I1116;    { 1280x1024x16 }
{-------[ Nastavenie palety ]--------------------------------------/////-----}
  procedure SetVGAPalette(PalBuf : DacPalette256);
{-[ IMPLEMENTATION ]-----------------------------------------------/////-----}
 implementation
  uses Crt, Dos, Graph, Strings;
{-----[ Vnutorne typy pre informacie o grafickej karte ]-----------/////-----}
  type VModes=array[1..238]of word;
       InfoBuffer=record
                   VESASig:array[1..4]of char;
                   VESAVer_Hi:byte;
                   VESAVer_Lo:byte;
                   Manufacturer:pchar;
                   Flag:longint;
                   VideoModes:^vmodes;
                   FreeSpace:array[1..119]of word;
                  end;
     P_InfoBuffer=^InfoBuffer;
{-----[ Vnutorne typy pre informacie o grafickom mode ]------------/////-----}
     ModeInfoBuffer=record
                     ModeFlag:word;
                     NotUsed1:array[1..16]of byte;
                     XResolution:word;
                     YResolution:word;
                     NotUsed2:array[1..3]of byte;
                     BitesPerDot:byte;
                     NotUsed3:array[1..230]of byte;
                    end;
     P_ModeInfoBuffer=^ModeInfoBuffer;

{-----[ Procedury pouzivane na detekciu gr. karty ]----------------/////-----}
{-------[ Ziska informacie o gr. karte ]---------------------------/////-----}
 function GetVESAInfo(var P:InfoBuffer):boolean;
  {-----[ Parametre ]-------------------------/////-----}
  { P = record, do ktoreho napiseme info                }
  {-----[ Vystup ]----------------------------/////-----}
  { Bola funkcia vykonana uspesne? TRUE=ano.            }
  var r:registers;
  begin
   r.ah:=$4f;
   r.al:=$00;
   r.es:=seg(p);
   r.di:=ofs(p);
   intr($10,r);
   getvesainfo:=(r.al=$4f)and(r.ah=$00);
  end;

{-------[ Ziska informacie o gr. mode ]----------------------------/////-----}
 function GetModeInfo(Mode:word;var P:ModeInfoBuffer):boolean;
  {-----[ Parametre ]-------------------------/////-----}
  { Mode = cislo modu                                   }
  { P    = record, kam zapiseme info o mode             }
  {-----[ Vystup ]----------------------------/////-----}
  { Bola funkcia vykonana uspesne? TRUE=ano.            }
  var r:registers;
  begin
   r.ah:=$4f;
   r.al:=$01;
   r.cx:=mode;
   r.es:=seg(p);
   r.di:=ofs(p);
   intr($10,r);
   getmodeinfo:=(r.ah=$00)and(r.al=$4f);
  end;

{-------[ Testuje, ci je gr. mod jeden z hladanych ]---------------/////-----}
 function Je_To_Nas_Mod(X,Y,B:word;PMI:ModeInfoBuffer):boolean;
  {-----[ Parametre ]-------------------------/////-----}
  { X,Y = testovane rozlisenie                          }
  { B   = testovana farebna hlbka (pocet bitov/pixel)   }
  { PMI = testovany graficky mod                        }
  {-----[ Vystup ]----------------------------/////-----}
  { Ma testovany gr. mod zadane rozlisenie a 256        }
  { farieb? TRUE=ano.                                   }
  begin
   {   X-rozlisenie            Y-rozlisenie              }
   if (pmi.xresolution=x)and(pmi.yresolution=y)and
   {   pocet farieb          graficky mod                }
      (pmi.bitesperdot=b)and((pmi.modeflag and 16)=16)and
   {   farebny mod               monitor je ok           }
      ((pmi.modeflag and 8)=8)and((pmi.modeflag and 1)=1) then
   {  vrati true, ak je to nas mod}
      je_to_nas_mod:=true else je_to_nas_mod:=false;
  end;

 procedure Nuluj(var VI:VESAInfo);
  begin
   VI.VESAVer_HI:=0;
   VI.VESAVer_LO:=0;
   VI.Manufacturer:='Card was not detected yet/correctly.';
   VI.Modes:=0;
  end;

{-------[ Zisti potrebne informacie o grafickej karte ]------------/////-----}
 function DetectVESA(var VI:VESAInfo):boolean;
  {-----[ Parametre ]-------------------------/////-----}
  { VI  = record, kam zapiseme info o karte a modoch    }
  {-----[ Vystup ]----------------------------/////-----}
  { Nasli sme v pocitaci graficku kartu VESA?           }
  { TRUE=ano.                                           }
  var a:word;
      gd,gm:integer;
      pmi:modeinfobuffer;
      pi:infobuffer;
  begin
   nuluj(vi);
   {je to VESA?}
   if getvesainfo(pi)and(pi.vesasig[1]+pi.vesasig[2]+pi.vesasig[3]+
                                       pi.vesasig[4]='VESA')then begin
    {je tam nejaka VESA, aka?}
    vi.vesaver_lo:=pi.vesaver_lo;
    vi.vesaver_hi:=pi.vesaver_hi;
    vi.manufacturer:=strpas(pi.manufacturer);

    a:=1;
    {Pre vsetky mody, alebo aspon vsetky do 238}
    while ((a<=238)and(pi.videomodes^[a]<>$ffff)) do begin
     {ak sa ti podari ziskat info o mode,}
     if getmodeinfo(pi.videomodes^[a],pmi) then begin
      {a este k tomu aj info o rozliseni, farbach, usw}
      if (pmi.modeflag and 2)=2 then begin
       {info o rozliseni mame, pouzijeme ho}
       {800x600x16}
       if je_to_nas_mod(800,600,4,pmi) then vi.modes:=vi.modes or $20;
       {1024x768x16}
       if je_to_nas_mod(1024,768,4,pmi) then vi.modes:=vi.modes or $40;
       {1280x1024x16}
       if je_to_nas_mod(1280,1024,4,pmi) then vi.modes:=vi.modes or $80;
      end else begin
      {info o rozliseni nemame, skusime pouzit cisla modov}
       if pi.videomodes^[a]=$102 then vi.modes:=vi.modes or $20;
       if pi.videomodes^[a]=$06a then vi.modes:=vi.modes or $20;
       if pi.videomodes^[a]=$104 then vi.modes:=vi.modes or $40;
       if pi.videomodes^[a]=$106 then vi.modes:=vi.modes or $80;
      end;
     end;
     inc(a);
    end;
   detectvesa:=true
   end else detectvesa:=false; {Karta nepodporuje VESA}
  end;

{-----[ Procedura, ktora detekuje VESA + optimalny mod ]-----------/////-----}
  {$F+}
  function DetectVESA16: integer;
  {-----[ Parametre ]-------------------------/////-----}
  { Ziadne                                              }
  {-----[ Vystup ]----------------------------/////-----}
  { Cislo optimalneho modu alebo grError ak nie je      }
  { pritomna VESA kompatibilna graficka karta.          }
   begin
    if DetectVesa(svgainfo) then begin
     if ((svgainfo.modes and $20)=$20) then DetectVESA16:=0;
     if ((svgainfo.modes and $40)=$40)and(MaxVESAMode>=SVGA1024x768x16)
        then DetectVESA16:=1;
     if ((svgainfo.modes and $80)=$80)and(MaxVESAMode>=SVGA1280x1024x16)
        then DetectVESA16:=2;
    end else DetectVESA16:=grError;
   end; { DetectVESA16 }
   {$F-}

{-----[ Inicializacia gr. modu kompatibilna so SVGA256 1.x ]-------/////-----}
  procedure Initialize(gm:integer;pd:string);
  {-----[ Parametre ]-------------------------/////-----}
  { GM = Cislo modu                                     }
  { PD = Cesta k suboru VESA16.BGI                      }
   var PathToDriver:string;
       ErrorCode:integer;
   begin
    DirectVideo:=False;
    PathToDriver:=PD;
    repeat
     InitGraph(VESA,GM,PathToDriver);
     ErrorCode:=GraphResult;
     if ErrorCode<>grOK then begin
      Writeln('Graphics error: ', GraphErrorMsg(ErrorCode));
      if ErrorCode=grFileNotFound then begin
       Writeln('Enter full path to BGI driver or type <Ctrl-Break> to quit:');
       Readln(PathToDriver);
       Writeln;
      end else Halt(1);
     end;
    until ErrorCode=grOK;
    setcolor(white);
   end; { Initialize }

{-----[ Nastavenie palety ]----------------------------------------/////-----}
  procedure SetVGAPalette(PalBuf:DacPalette256);
  {-----[ Parametre ]-------------------------/////-----}
  { PalBuf = Nova paleta                                }
   var Reg:Registers;
   begin
    reg.ax := $1012;
    reg.bx := 0;
    reg.cx := 256;
    reg.es := Seg(PalBuf);
    reg.dx := Ofs(PalBuf);
    intr($10,reg);
   end;

{-----[ Rychla inicializacia gr. modov ]---------------------------/////-----}
  procedure I8616;
   begin
    initialize(0,bgi);
   end;

  procedure I1716;
   begin
    initialize(1,bgi);
   end;

  procedure I1116;
   begin
    initialize(2,bgi);
   end;

  begin
  MaxVESAMode:=SVGA1024x768x16;
  VESA:=DetectVESA16;
  VESA:=InstallUserDriver('VESA16', @DetectVESA16);
  if VESA=grError then VESA:=Detect;
 end.
